from django.db import models
from django.test import TestCase
from django.utils.baseconv import base64
from django_base64field.fields import Base64Field

class Planet(models.Model):
    ek = Base64Field()
    name = models.CharField(
        default='Fucker',
        max_length=103
    )


class Continent(models.Model):
    ek = Base64Field()
    name = models.CharField(
        default='Suckers!',
        max_length=13
    )
    planet = models.ForeignKey(Planet, to_field='ek')


class Helper(models.Model):
    """
    base64 encoded value won't be available at first time creation.
    It can ve accessible by getting the object from database after creation
    mean when it get saved completely, But what if we don't want to get our base64
    encoded key from our sweet model by retrieving it again from database?

    It's easy, efficient, holly and molly!
    """
    ek = Base64Field()


    def _ek(self):
        if self.ek: return self.ek

        if (not self.ek) and (self.pk):
            return base64.encode(self.pk)

        return self.ek


class TestBase64Field(TestCase):

    def test_field_is_none_on_creation(self):
        """
         After Planet has created, `planet.ek` won't be available.
        because `planet.ek` will be generated after `planet` has been saved.
        """
        planet = Planet.objects.create()

        self.assertIn(planet.ek, ['', None])
        self.assertIsNotNone(planet.pk)

    def test_field_not_none_after_saved(self):
        """
         After `Planet` has been created, `Planet.ek` will be available
        on next time of getting `planet` from database.

        Why?
        ----
             Because after `planet` get created, `post_save` signal of `Planet` will
            emitted, This is where `ek` field will be set from ``base64``
            encoded value based on `pk`.
        """
        planet = Planet.objects.create()
        base64_key = base64.encode(planet.pk)
        same_planet_but_fresh = Planet.objects.get(pk=planet.pk)

        self.assertEqual(same_planet_but_fresh.ek, base64_key)
        self.assertNotEqual(planet.ek, same_planet_but_fresh.ek)

    def test_field_for_fk_none_on_creation(self):
        """
         I just created a `Planet`, As you already know, `planet.ek` is not
        available on `planet` at the moment. It will be available on next
        retrieving from database. So `planet` can't be assigned for creating
        new `Continent` as FK on `Continent.planet`.
        """
        planet = Planet.objects.create()

        continent = None

        try:
            continent = Continent.objects.create(planet=planet)
        except:
            pass
        finally:
            self.assertIsNone(continent)

    def test_field_for_fk_not_none_after_creation(self):
        """
         So I want to create new `Continent` for my new shiny funky punky ass
        `Planet` that I've just created, It's easy, just retrieving new
        planet from database to make `ek` available for it, Then it will be
        fucking freaking ok to assign the fresh `Planet` to
        our sucker `Continent`.
        """
        planet = Planet.objects.create()
        same_planet_but_fresh = Planet.objects.get(pk=planet.pk)
        continent = Continent.objects.create(planet=same_planet_but_fresh)

        self.assertIsNotNone(continent)
        self.assertIsInstance(continent, Continent)
        self.assertIn(continent.ek, ['', None])

        continent_pk = continent.pk
        same_continent_but_fresh = Continent.objects.get(pk=continent_pk)


        self.assertNotEqual(same_continent_but_fresh.ek, continent.ek)
        self.assertEqual(same_continent_but_fresh.ek, base64.encode(continent_pk))

    def test_field_with_helper_defined(self):
        """
         Look at `Helper` Model. Won't be any headache for retrieving the
        object after creation from database anymore.
        """
        hell = Helper.objects.create()

        #  On `hell` instance there is no `ek` available
        # because It will be available on the next retrieving of `Helper`
        self.assertIn(hell.ek, ['', None])
        self.assertNotEqual(hell.ek, hell._ek())

        #  But if `ek` is need it immediately for any reason
        # `ek` will be available on `_ek()` method.
        # Without any refreshing/retrieving `hell` instance again
        # from database
        self.assertIsNotNone(hell._ek())
        self.assertNotIn(hell._ek(), ['', None])




from django.db.models import signals
from django.db.models.fields import CharField

class Base64Field(CharField):
    """
     Base64Field is useful where you need a base64 encoded value from
    model's Primary Key a.k.a PK which is available on every django
    application model by default. Sine `base64` encoder works with
    `integer` value then PK should be also `integer`, Fortunately
    again `PK` field is `integer` by nature.

     When a model get saved, `post_save` signal will be emitted,
    This is where a base64 encoded key will be generated/encoded
    from model's `PK`, Then model will get **updated** not saved again.
    this operation happens just one the first time model get saved.
    In next time model get saved or updated `base64` won't get generated.

    Usage
    -----
    >>> from django.db import models
    >>> from django_base64field.fields import Base64Field
    >>>
    >>> class MyModelianto(models.Model):
    >>>     ek = Base64Field()
    >>>
    >>> modelia = MyModelianto.objects.create(pk=314159265358979323)
    >>> modelia.ek
    >>> u''
    >>> refreshed_modelia = MyModelianto.objects.get(pk=modelia.pk)
    >>> modelia.ek
    >>> u'HS7Y_sdg3x'
    """

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 255)
        kwargs['blank'] = kwargs.get('blank', True)
        kwargs['editable'] = kwargs.get('editable', False)
        kwargs['default']  = kwargs.get('default', '')
        kwargs['unique'] = True
        super(Base64Field, self).__init__(*args, **kwargs)

    def contribute_to_class(self, cls, name):
        super(Base64Field, self).contribute_to_class(cls, name)
        cls._base64field_name = name
        signals.post_save.connect(self.generate_encoded_pk, cls)

    def generate_encoded_pk(self, sender, **kwargs):
        instance = kwargs['instance']
        field_name = instance._base64field_name

        if getattr(instance, field_name) in ['', None]:
            from django.utils.baseconv import base64

            obj_pk = instance.pk

            sender._default_manager.filter(pk=obj_pk).update(
                **{field_name:base64.encode(obj_pk)}
            )

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^django_base64field\.fields\.Base64Field"])
except ImportError:
    # No error should silently get passed my ASS!
    pass



from distutils.core import setup

setup(
    name='django-base64field',
    version='0.9',
    packages=['django_base64field'],
    url='https://github.com/Alir3z4/django-base64field',
    license=open('LICENSE').read(),
    author='Alireza Savand',
    author_email='alireza.savand@gmail.com',
    install_requires=['django',],
    description='A motherfucking django model field to bring base64 encoded'
                ' key to models.',
    long_description=open('README.rst').read(),
    keywords=[
        'django',
        'base64',
        'field'
    ],
    platforms='OS Independent',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Framework :: Django',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development'
    ],

)
